import glob
import os
import re
import subprocess
import sys
import traceback
from multiprocessing import Pool
from pathlib import Path

import typer
from plumbum import local
from gosu import cmds

app = typer.Typer()
app.add_typer(cmds.ci.app, name="ci")
app.add_typer(cmds.python.app, name="python")
app.add_typer(cmds.django.app, name="django")


def get_project_root():
    stack = reversed(traceback.extract_stack())

    for frame in stack:
        try:
            path = Path(frame.filename).resolve()
        except OSError:
            # some frames have names that cannot be treated as file paths
            continue

        # packages installed using pip are stored in the 'site-packages'
        # if from_root is called from a package, we can quickly find the root directory
        posix_like = path.as_posix()
        if "site-packages" in posix_like:
            root_path = Path(re.compile(r".*site-packages/.*?/").findall(posix_like)[0])
            # but we ignore 'from_root' package
            if root_path.name != "from_root":
                return root_path

        while path.parents:
            if (
                (path / ".git").exists()
                or (path / ".project-root").exists()
                or (path / "setup.py").exists()
            ):
                return path

            path = path.parent

    raise FileNotFoundError(
        'There is neither ".git" directory nor ".project-root" file nor "setup.py", cannot detect root folder'
    )


def safe_run(cmd, **kwargs):
    r = subprocess.run(cmd, capture_output=True, **kwargs)
    if r.returncode != 0:
        sys.stdout.write(r.stdout.decode("utf8"))
        sys.stdout.write(r.stderr.decode("utf8"))
        raise Exception("error with", cmd)


def pbuild(file):
    project_base = os.path.abspath(local.cwd + "/" + file.split("/")[0])
    project_name = project_base.split("/")[-1]
    print(f"build {project_base}")  # noqa
    safe_run(["gosu", "venv"], cwd=project_base)
    safe_run(["gosu", "build"], cwd=project_base)
    safe_run(
        ["cp", glob.glob(project_base + "/dist/*.tar.bz2")[0], "../dist"],
        cwd=project_base,
    )
    safe_run(
        [
            "cp",
            glob.glob(project_base + "/dist/requirements_compiled.txt")[0],
            f"../dist/requirements_{project_name}.txt",
        ],
        cwd=project_base,
    )


def build_all():
    safe_run(["mkdir", "-p", "dist"])
    with Pool(3) as p:
        p.map(pbuild, glob.glob("**/setup.py"))


def import_from_path(path, module_name):
    import importlib.util
    spec = importlib.util.spec_from_file_location(module_name, path)
    foo = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(foo)
    return foo


def find_local_gosu_cmds():
    cmds_path = os.getcwd() + '/cmds.py'
    if not os.path.isfile(cmds_path):
        return

    import_from_path(cmds_path, "gosu.user.cmds")


find_local_gosu_cmds()



import typer
import os
import re
import semver
import subprocess


app = typer.Typer()


def git(*args):
    return subprocess.check_output(["git"] + list(args))


def get_version():
    try:
        return git("describe", "--tags").decode().strip()
    except subprocess.CalledProcessError:
        return "1.0.0"


@app.command()
def push_repo():
    git("remote", "set-url", "--push", "origin", re.sub(r'.+@([^/]+)/', r'git@\1:', os.environ["CI_REPOSITORY_URL"]))
    git("push", '-o', 'ci.skip', "origin", get_version())


@app.command()
def bump_version():
    v = get_version()
    n = semver.bump_patch(v)
    if '-' not in v:
        return

    print(f"bump from {v} to {n}")
    git("tag", n)



from gosu.tools import env_from_sourcing, manage_env_vars, _run_venv
from plumbum import FG, local
import glob
from typing import List
import typer

app = typer.Typer()


@app.command()
def sh():
    manage_env_vars()
    for k, v in env_from_sourcing(".venv/bin/activate").items():
        local.env[k] = v
    local.get(local.env["SHELL"].split("/")[-1]) & FG


@app.command()
def venv(env: str = typer.Argument("dev")):
    manage_env_vars()
    local.cmd.python3["-m", "venv", ".venv"] & FG
    _run_venv(["pip", "install", "-U", "pip", "wheel"])
    _run_venv(["pip", "install", "-r", f"requirements/{env}.txt"])


@app.command()
def run(cmds: List[str]):
    _run_venv(cmds)


def _get_pkg_pyfiles():
    files = list(
        filter(
            lambda e: all(
                [not (x in e) for x in ["node_modules", "migrations", "build"]]
            ),
            glob.glob("**/*.py", recursive=True),
        )
    )
    print(files)  # noqa
    return files


def fix():
    local.cmd.pyupgrade.__getitem__(["--py38-plus", *_get_pkg_pyfiles()]) & FG
    local.cmd.isort.__getitem__(["--profile", "black", *_get_pkg_pyfiles()]) & FG
    local.cmd.black.__getitem__([*_get_pkg_pyfiles()]) & FG



import unittest

from gosu.gosu import env_from_sourcing, get_project_root


class TestBasicMethods(unittest.TestCase):
    def test_get_env_from_sourcing(self):
        env = env_from_sourcing(f"{get_project_root()}/.venv/bin/activate")
        self.assertGreater(len(env), 0)


if __name__ == "__main__":
    unittest.main()



import subprocess
import json
from typing import List
from plumbum import FG, local, ProcessExecutionError


def env_from_sourcing(source_path, include_unexported_variables=True):
    source = "{}source {}".format(
        "set -a && " if include_unexported_variables else "", source_path
    )
    dump = 'python3 -c "import os, json; print(json.dumps(dict(os.environ)))"'
    with subprocess.Popen(
        ["/bin/bash", "-c", f"{source} && {dump}"], stdout=subprocess.PIPE
    ) as pipe:
        return json.loads(pipe.stdout.read())


def manage_env_vars():
    try:
        from dotenv import dotenv_values

        env_vars = dotenv_values(".env")
        for name, value in env_vars.items():
            local.env[name] = value
        return env_vars
    except ModuleNotFoundError:
        print("dotenv not found, dotenv autoloading disabled")  # noqa


def get_project():
    if "DJANGO_SETTINGS_MODULE" not in local.env:
        return "example"
    else:
        return local.env["DJANGO_SETTINGS_MODULE"].split(".")[0]


def _run_venv(cmds: List[str]):
    manage_env_vars()
    print(cmds)  # noqa
    try:
        local.get(f".venv/bin/{cmds[0]}")[cmds[1:]] & FG
    except ProcessExecutionError as e:
        exit(1)




from typing import List
from gosu.tools import get_project, manage_env_vars, _run_venv
from gosu.cmds.python import fix
import typer
import os
from plumbum import FG, local, ProcessExecutionError


app = typer.Typer()


def _pm(cmds: List[str]):
    manage_env_vars()
    _run_venv(["python", "-m", f"{get_project()}.manage", *cmds])


@app.command()
def pm(ctx: typer.Context):
    _pm(ctx.args)


@app.command()
def migrate():
    _pm(["makemigrations"])
    _pm(["migrate"])


@app.command()
def test():
    fix()
    manage_env_vars()

    local.env["DEFAULT_CACHE"] = "locmemcache://"
    local.env["QUEUE_CACHE"] = "locmemcache://"
    rcfile = f"--rcfile={os.path.dirname(__file__)}/../.coveragerc"
    _run_venv(
        [
            "coverage",
            "run",
            rcfile,
            "--parallel-mode",
            "--concurrency=multiprocessing",
            f"{get_project()}/manage.py",
            "test",
            ".",
            get_project(),
            # "--parallel=3",
        ]
    )
    _run_venv(["coverage", "combine", rcfile])
    _run_venv(["coverage", "report", rcfile])
    try:
        local.cmd.flake8["--config", f"{os.path.dirname(__file__)}/../.flake8"] & FG
    except ProcessExecutionError as e:
        exit(1)


@app.command()
def build():
    test()
    local.cmd.python3["setup.py", "sdist", "--formats=bztar"] & FG
    local.cmd.python3["setup.py", "clean"] & FG


@app.command()
def notebook():
    local.env["DJANGO_ALLOW_ASYNC_UNSAFE"] = True
    _pm(["shell_plus", "--notebook"])



from setuptools import setup, find_packages
import versioneer


setup(
    name="gosu",
    description='the extensible taskrunner',
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        "plumbum",
        "typer",
        "python-dotenv",
        "semver",
        "pytest",
        "pyupgrade",
        "isort",
        "black",
        "flake8",
        "mccabe",
        "flake8-comprehensions",
        "flake8-print",
        "flake8-cognitive-complexity",
    ],
    license_files=("LICENSE",),
    url='https://gitlab.com/tuhls/gosu',
    author='Herbert Rusznak (tlb)',
    author_email='herbert.rusznak@gmail.com',
    license='MIT',
    entry_points={"console_scripts": ("gosu = gosu.main:app",)},
)
